# info.py.in
#
# Copyright 2025 Qwery
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0-or-later


APP_ID = 'dev.qwery.AddWater'
VERSION = '1.3'
PROFILE = 'default'
PREFIX = '/dev/qwery/AddWater'

# NOTE: Meson booleans are strings! Not native python bools. Could convert them here if that makes sense.
FORCE_BG = 'False'
MOCK_API = 'False'



# About Menu
ISSUE_TRACKER = 'https://github.com/largestgithubuseronearth/addwater/issues/'
WEBSITE = 'https://addwater.qwery.dev/'